export default function Query(context, query) {
    /**
     * Determine if soft deleted models should be filtered exclusively.
     *   true  = only soft deletes
     *   false = include soft deletes
     *   null  = exclude soft deletes
     */
    query.prototype.softDeleteSelectFilter = null;
    /**
     * Constraint includes soft deleted models.
     */
    query.prototype.withTrashed = function () {
        this.softDeleteSelectFilter = false;
        return this;
    };
    /**
     * Constraint restricts to only soft deleted models.
     */
    query.prototype.onlyTrashed = function () {
        this.softDeleteSelectFilter = true;
        return this;
    };
    /**
     * Deprecated alias method for `onlyTrashed`.
     * @deprecated since v1.2.0
     */
    query.prototype.trashed = function () {
        /* istanbul ignore next */
        if (process.env.NODE_ENV !== 'production') {
            console.warn('The `trashed` method has been deprecated. Please use `onlyTrashed`.');
        }
        return this.onlyTrashed();
    };
    /**
     * Process the model(s) to be soft deleted.
     */
    query.prototype.softDelete = function (condition) {
        var _a;
        var _b = context.createConfig(this.model.softDeleteConfig), key = _b.key, flagName = _b.flagName, mutator = _b.mutator;
        var value = Date.now();
        value = typeof mutator === 'function' ? mutator(value) : value;
        var data = (_a = {},
            _a[key] = value,
            _a[flagName] = true,
            _a);
        if (Array.isArray(condition)) {
            // Array of primary keys
            if (!this.model.isCompositePrimaryKey()) {
                return this.model.update({
                    data: data,
                    where: function (record) { return condition.includes(record[record.$primaryKey()]); }
                });
            }
            // Array of composite primary keys
            if (condition.some(function (value) { return Array.isArray(value); })) {
                var keys_1 = condition
                    .map(function (key) { return Array.isArray(key) && JSON.stringify(key); })
                    .filter(Boolean);
                return this.model.update({
                    data: data,
                    where: function (record) { return keys_1.includes(record.$id); }
                });
            }
        }
        return this.model.update({ data: data, where: condition });
    };
    /**
     * Fetch all soft deletes from the store.
     */
    query.prototype.allTrashed = function () {
        return this.newQuery()
            .onlyTrashed()
            .get();
    };
    /**
     * Patch any new queries so that sub queries, such as relation queries,
     * can respect top level modifiers. For many-to-many relations, due to core
     * API limitations, we're forced to pass-through intermediate models.
     */
    var newQuery = query.prototype.newQuery;
    query.prototype.newQuery = function (entity) {
        var patchedQuery = newQuery.call(this, entity);
        // Only patch queries that are loading relations.
        var loadables = Object.keys(this.load);
        if (loadables.length > 0) {
            patchedQuery.softDeleteSelectFilter = this.softDeleteSelectFilter;
            if (entity && entity !== this.entity && this.model.hasPivotFields()) {
                var fields = this.model.pivotFields().reduce(function (fields, field) {
                    Object.keys(field)
                        .filter(function (entity) { return loadables.includes(entity); })
                        .forEach(function (entity) {
                        fields.push(field[entity].pivot.entity);
                    });
                    return fields;
                }, []);
                // Release an entity that is an intermediate to a loadable relation.
                if (fields.includes(entity)) {
                    patchedQuery.softDeleteSelectFilter = false;
                }
            }
        }
        return patchedQuery;
    };
    /**
     * Fetch all soft deletes from the store and group by entity.
     */
    query.allTrashed = function (store) {
        var _this = this;
        var database = store.$db();
        var models = database.models();
        return Object.keys(models).reduce(function (collection, entity) {
            collection[entity] = new _this(store, entity).onlyTrashed().get();
            return collection;
        }, {});
    };
    /**
     * Global select hook prevents soft deleted models from being selected unless
     * queries are explicity chained with `withTrashed` or `onlyTrashed`.
     */
    query.on('beforeSelect', function (models) {
        var _this = this;
        return models.filter(function (model) {
            // Only soft deletes
            if (_this.softDeleteSelectFilter === true) {
                return model.$trashed();
            }
            // Include soft deletes
            if (_this.softDeleteSelectFilter === false) {
                return models;
            }
            // Exclude soft deletes
            return !model.$trashed();
        });
    });
}
//# sourceMappingURL=Query.js.map